<?php
namespace ID3\DLP;

class Rest{
    private $_routes = [];
    private $_locale = null;

    function __construct(){
        add_action( 'rest_api_init', [$this,'init']);
    }
    
    public function init() {
        $this->registerRoutes();

    }

    function registerRoutes(){
        
        // remove_filter( 'rest_pre_serve_request', 'rest_send_cors_headers' );
        // add_filter( 'rest_pre_serve_request', function( $value ) {
        //     header( 'Access-Control-Allow-Origin: *' );
        //     header( 'Access-Control-Allow-Methods: GET' );
        //     header( 'Access-Control-Allow-Credentials: true' );
        //     header( 'Access-Control-Expose-Headers: Link', false );

        //     return $value;
        // } );


        $this->addRoute('/categories', [$this, 'getCategories'],[
            'methods' => \WP_REST_Server::READABLE,
            'args' => [
                'locale' => [
                    'required' => false,
                    'type' => 'string',
                    'description' => 'Filter categories by language',
                ]
            ]
        ]);
        $this->addRoute('/categoryCount', [$this, 'getCategoryCount']);

        $this->addRoute('/supported-languages', [$this, 'getSupportedLanguages']);

        $this->addRoute('/registrations', [$this, 'getRegistrations']);

        
        
        $this->addRoute('/registration', [$this, 'removeRegistration'],[
            'methods' => \WP_REST_Server::DELETABLE,
            'args' => [
                'slave_id' => [
                    'required' => true,
                    'type' => 'string',
                    'description' => 'Slave site id making the request (use domain)',
                ]
            ]
        ]);
        $this->addRoute('/registration/deactivate', [$this, 'deactivateRegistration'],[
            'methods' => \WP_REST_Server::CREATABLE,
            'args' => [
                'slave_id' => [
                    'required' => true,
                    'type' => 'string',
                    'description' => 'Slave site id making the request (use domain)',
                ]
            ]
        ]);
        $this->addRoute('/registration/activate', [$this, 'activateRegistration'],[
            'methods' => \WP_REST_Server::CREATABLE,
            'args' => [
                'slave_id' => [
                    'required' => true,
                    'type' => 'string',
                    'description' => 'Slave site id making the request (use domain)',
                ]
            ]
        ]);
        


        $this->addRoute('/salvage', [$this, 'getArticles'],[
            'methods' => \WP_REST_Server::CREATABLE,
            'args' => [
                'slave_id' => [
                    'required' => true,
                    'type' => 'string',
                    'description' => 'Slave site id making the request (use domain)',
                ],
                'limit_date' => [
                    'required' => false,
                    'validate_callback' => function($param, $request, $key) {
                        if($param == null) return true;
                        return $param != '';
                      },
                    'description' => 'Lower date boundary for salvaging',
                ],
                'filters' => [
                    'required' => false,
                    'type' => 'array',
                    'description' => 'Category filters',
                ],
                'locale' => [
                    'required' => false,
                    'type' => 'string',
                    'description' => 'Filter articles by language',
                ],
                'sync_method' => [
                    'required' => false,
                    'type' => 'string',
                    'description' => 'Method used for syncing. Default to Incremental',
                ],
            ]
        ]);

        

        foreach ($this->_routes as $path => $routeDef) {
            register_rest_route( 'id3-dlp',$path, $routeDef);
        }
    }
    function addRoute($path, $callback, $options=null){
        $routeDef = [
            
                'methods' => \WP_REST_Server::READABLE,
                'callback' => $callback,
                'args' => [],
                'permission_callback' => function(){return true;}
            
        ];

        if($options != null){
            if (isset($options['methods'])){
                $routeDef['methods'] = $options['methods'];
            }
            
            if (isset($options['args'])){
                $routeDef['args'] = $options['args'];
            }
        } 
        $this->_routes[$path] = $routeDef; //register_rest_route( 'id3-rest',$path, $routeDef);
    }

    function switchLocale($lang){
        if(Main::is_plugin_active('sitepress')){
            $this->_locale = apply_filters( 'wpml_current_language', null );
            do_action( 'wpml_switch_language', $lang );

            if($lang != apply_filters( 'wpml_current_language', null )){
				global $sitepress;
				$sitepress->switch_lang($lang);
			}
			
			   
			if($lang != apply_filters( 'wpml_current_language', null )){
				error_log('failed to switch locale to ' . $lang);	
                return false;
			}

            return true;
        }
        

        return false;
    }
    function resetLocale(){
        if($this->_locale != null){
            do_action( 'wpml_switch_language', $this->_locale );
            $this->_locale = null;
        }
    }

    function getRegistrations(){
        $slaveMan = new Slave();
        $list = $slaveMan->list();
        $result = [];

        foreach ($list as $id) {
            $slave = new Slave($id);
            $slave->load();

            $result[] = $slave;
        }

        return $result;

    }

    function removeRegistration($request){
        $slave_id       = $request->get_param('slave_id');

        $slave = new Slave($slave_id);
        return $slave->delete();
    }

    function deactivateRegistration($request){
        $slave_id       = $request->get_param('slave_id');

        $slave = new Slave($slave_id);
        $slave->load();
        $slave->active = false;
        $slave->update();

        return true;
    }

    function activateRegistration($request){
        $slave_id       = $request->get_param('slave_id');

        $slave = new Slave($slave_id);
        $slave->load();
        $slave->active = true;
        $slave->update();

        return true;
    }



    function getSupportedLanguages(){
        $result = ['fr','en'];

        if(Main::is_plugin_active('sitepress')){
            $languages = apply_filters( 'wpml_active_languages', NULL, 'orderby=id&order=desc' );
            if ( !empty( $languages ) ) {
                $result = [];
                foreach( $languages as $l ) {
                    $result[] = $l['language_code'];
                    // if ( !$l['active'] ) echo ' <a href="' . esc_url( $l['url'] ) . '">';
                    // echo '<img src="' . esc_url ( $l['country_flag_url'] ) . '" height="12" alt="' . esc_attr( $l['language_code'] ) . '" width="18" />';
                    // if ( !$l['active'] ) echo '</a> ';
                }
            }
        }
        if(function_exists('pll_the_languages')){
            $languages = pll_the_languages(['raw'=>1]);

            if ( !empty( $languages ) ) {
                $result = [];
                foreach( $languages as $l ) {
                    $result[] = $l['id'];
                }
            }
        }

        return $result;
    }

    function getCategoryCount(){
        $categories = get_categories(['hide_empty' => false]);
        
        if(empty($categories)) return 0;
        return(count($categories));
    }
    function getCategories($request){
        $locale = $request->get_param('locale');
        $supportLocaleFilter = false;
        if(!empty($locale)){
            $supportLocaleFilter = $this->switchLocale($locale);
        }

        if ( Main::is_plugin_active('sitepress') ) {
            global $sitepress;

            //     remove_filter( 'get_terms_args', array( $sitepress, 'get_terms_args_filter' ) );
            //     remove_filter( 'get_term', array( $sitepress, 'get_term_adjust_id' ) );
            remove_filter( 'terms_clauses', array( $sitepress, 'terms_clauses' ) );
        }        

        $categories = get_categories(['hide_empty' => false]);
        
        $results = [];
        foreach ($categories as $cat) {
            $sp_args = ['element_id' => $cat->cat_ID, 'element_type' => 'category'];
            $localeCode = apply_filters( 'wpml_element_language_code', null, $sp_args );
            $cat->lang = $localeCode;

            if($supportLocaleFilter && !empty($locale)){
                if( $localeCode == $locale){
                    $results[] = $cat;
                }
            }
            else{
                // If no locale was pass, add the category. 
                $results[] = $cat;
            }
        }

        //error_log($locale . ' category: ' . json_encode($categories));
        
        $this->resetLocale();
        return $results;
    }

    function getArticles($request){
        $response       = ['success' => false, 'message' => ''];
        $slave_id       = $request->get_param('slave_id');
        $limit_date     = $request->get_param('limit_date');
        $filters        = $request->get_param('filters');
        $method         = $request->get_param('sync_method');
        $locale         = $request->get_param('locale');
        
        if($method == null) $method = 'incremental';
        $supportLocaleFilter = false;

        if(!empty($locale)){
            $supportLocaleFilter = $this->switchLocale($locale);
        }

        $slave = new Slave($slave_id);
        $newRegistration = $slave->load();

        if($newRegistration === null) {
            $response['message'] = 'Registration failed for ' . $slave_id;
            return $response;
        }

        // check slave access
        if(!$slave->active) {
            $response['message'] = $slave_id . ' has been deactivated';
            return $response;
        }
        

        // force last_sync date to slave when it's provide by the request
        if($limit_date != null){
            $slave->last_sync = $limit_date;
        }
        if($method == 'full'){
            $slave->compare_date = 'post_date';
        }

        
        // update slave's filters when request provide new filters
        if($filters != null){
            error_log('REST: getArticles(' . $locale . ') -> ' . json_encode($filters));
            $slave->set_filters($filters);
        }
        

        $articles = $slave->getNewArticles();
        error_log('REST: getArticles(' . $locale . ') @ raw => ' . json_encode($articles));
        $filteredArticles = $slave->applyArticleFilters($articles);
        error_log('REST: getArticles(' . $locale . ') @ filtered => ' . json_encode($filteredArticles));
        $slave->loadArticlesExtraMeta($filteredArticles['list']);
        
        // update Slave last_sync date to now
        $slave->last_sync = date("Y-m-d H:i:s e");


        // save Slave data
        $slave->update();

        $response['success'] = true;
        $response['articles'] =  $filteredArticles['list'];
        $response['filter_result'] = $filteredArticles['infos'];
        $response['new'] = $newRegistration;
        $response['tz'] = date_default_timezone_get();
        $response['slave'] = $slave;

        $this->resetLocale();

        //error_log('REST: getArticles(' . $locale . ') => ' . json_encode($response));
        return $response;
    }
}
