<?php
namespace ID3\DLP;

class Updater{

    private $sourceRepo;
    private $repo;
    private $slug; // plugin slug
    private $pluginData; // plugin data
    private $pluginFile; // __FILE__ of our plugin
    private $repoResult; // holds data from GitHub
    
    function __construct($pluginFile, $repo_name=null, $source='https://update.id-3.net'){
      
      add_filter( "pre_set_site_transient_update_plugins", array($this, "setTransientPluginUpdate") );      
      add_filter( "plugins_api", [$this, "setPluginInfo" ], 10, 3 );
      //add_filter( "upgrader_post_install", [$this, "postInstall"], 10, 3 );
      $siteRoot = get_bloginfo('url');
    
      $pluginFile = str_replace('\\','/', $pluginFile);
      $this->pluginUrl = $siteRoot . '/' . dirname(str_replace(str_replace('\\', '/', ABSPATH),'', $pluginFile));
      $this->pluginFile = $pluginFile;
      $this->sourceRepo = $source;
      $this->repo = ($repo_name==null) ? basename(dirname($pluginFile)) : $repo_name;
    }
  
    // Get information regarding our plugin from WordPress
    private function initPluginData() {
      $this->slug = plugin_basename( $this->pluginFile );
      $this->pluginData = get_plugin_data( $this->pluginFile );
    }
  
    function setTransientPluginUpdate($transient){
      // If we have checked the plugin data before, don't re-check
      if ( empty( $transient->checked ) ) {
        return $transient;
      }
  
      $this->initPluginData();
      //$plugin_info = get_site_transient('update_plugins');
      $pluginInfos = $this->getRepoPluginInfo();
      if(!$pluginInfos) return $transient;

      // Check the versions if we need to do an update
      if(isset($transient->checked[$this->slug])){
        if(version_compare( $pluginInfos->version, $transient->checked[$this->slug], '<=' )) return $transient;
      }
      
      // Update the transient to include our updated plugin data
      $obj = new PluginInfos();
      $obj->id = 'wp.org/plugins/' . $this->repo;
      $obj->slug = $this->repo;
      $obj->plugin = $this->slug;
      $obj->new_version = $pluginInfos->version;
      $obj->url = $this->pluginData["PluginURI"];
      $obj->package = $this->sourceRepo . $pluginInfos->download_link;;
      $obj->icons = [
        "1x" => $this->pluginUrl . '/assets/logo-128x128.jpg',
        "2x" => $this->pluginUrl . '/assets/logo-256x256.jpg'
      ];
      $obj->banners = [
        "low" =>  $this->pluginUrl . '/assets/banner-772x250.jpg',
        "high" => $this->pluginUrl . '/assets/banner-1544x500.jpg'
      ];
      $obj->banners_rtl = array();
  
      // Gets the tested version of WP if available
      $matches = null;
      preg_match( "/tested:\s([\d\.]+)/i", $pluginInfos->body, $matches );
      if ( ! empty( $matches ) ) {
          if ( is_array( $matches ) ) {
              if ( count( $matches ) > 1 ) {
                $obj->tested = $matches[1];
              }
          }
      }
      
      $transient->response[$this->slug] = $obj;
      return $transient;
    }
  
    function getRepoPluginInfo(){
      if( !empty($this->repoResult) ) return $this->repoResult;
      
      $url = $this->sourceRepo . "/infos/{$this->repo}/json";
      $callArgs = [];
  
      $infosRaw =  wp_remote_retrieve_body( wp_remote_get( $url, $callArgs ) );
      
      
      $this->repoResult = json_decode( $infosRaw );
      return $this->repoResult;
    }
  
    function setPluginInfo($plugins_api, $action, $args){
      
      // Get plugin & GitHub release information
      $this->initPluginData();
      
      
      $pluginSlug = isset($args->slug) ? $args->slug : '';
      // If nothing is found, do nothing
      // if ("plugin_information" !== $action) {
      //   return $plugins_api;
      // }
  
      if ($pluginSlug != $this->repo) {
        return $plugins_api;
      }
  
      if ( empty( $args->slug ) || $args->slug != $this->repo ) {
        return $plugins_api;
      }
  
      $repoInfos = $this->getRepoPluginInfo();
  
      // Add our plugin information
      $args->last_updated = $repoInfos->modify_date;
      $args->slug = $this->slug;
      $args->plugin_name  = $this->pluginData["Name"];
      $args->name  = $this->pluginData["Name"];
      $args->version = $repoInfos->version;
      $args->author = $this->pluginData["AuthorName"];
      $args->homepage = $this->pluginData["PluginURI"];
      $args->banners = [
        "high" => $this->pluginUrl . '/assets/banner-1544x500.jpg',
        "low" => $this->pluginUrl . "/assets/banner-772x250.jpg"
      ];
      
      // This is our release download zip file
      $downloadLink = $repoInfos->download_link;
      $args->download_link = $downloadLink;
  
      // We're going to parse the GitHub markdown release notes, include the parser
      require_once( plugin_dir_path( __FILE__ ) . 'class.parsedown.php' );
  
      // Create tabs in the lightbox
      if(! empty($repoInfos->sections)){
        $args = $repoInfos->sections;
      }
      else{
        $repoBody = explode('## Change log',$repoInfos->body);
        $changelogs = \ID3\Parsedown::instance()->parse( $repoBody[1]);
        $description = \ID3\Parsedown::instance()->parse( $repoBody[0]);
  
        $args->sections = [
          'description' => $description,
          'changelog' => substr($changelogs,0,strpos($changelogs,'requires:'))
        ];
      }
  
      // Gets the required version of WP if available
      $matches = null;
      preg_match( "/requires:\s([\d\.]+)/i", $repoInfos->body, $matches );
      if ( ! empty( $matches ) ) {
          if ( is_array( $matches ) ) {
              if ( count( $matches ) > 1 ) {
                  $args->requires = $matches[1];
              }
          }
      }
      
      // Gets the tested version of WP if available
      $matches = null;
      preg_match( "/tested:\s([\d\.]+)/i", $repoInfos->body, $matches );
      if ( ! empty( $matches ) ) {
          if ( is_array( $matches ) ) {
              if ( count( $matches ) > 1 ) {
                  $args->tested = $matches[1];
              }
          }
      }
  
  
      return $args;
    }
  
  
    // Use this method to rename folder upon installation
    // in case of a directory name mismatche, but it shouldn't be the case
    function postInstall($true, $hook_extra, $result){
  
      // remove the next line to actually do somethig
      return $result;
  
  
      // Get plugin information
      $this->initPluginData();
        
      // Remember if our plugin was previously activated
      $wasActivated = is_plugin_active( $this->slug );
  
      global $wp_filesystem;
      $pluginFolder = WP_PLUGIN_DIR . DIRECTORY_SEPARATOR . dirname( $this->slug );
      $wp_filesystem->move( $result['destination'], $pluginFolder );
      $result['destination'] = $pluginFolder;
  
      // Re-activate plugin if needed
      if ( $wasActivated ) {
        $activate = activate_plugin( $this->slug );
      }
  
      return $result;
    }
  }

class PluginInfos{
        
    public $id;
    public $plugin;
    public $slug;
    public $new_version;
    public $url;
    public $package;
    public $icons = [];
    public $banners = [];
    public $banners_rtl;

    public $tested;
    
}

// if(!class_exists('GitHubPluginUpdater')){
//     class GitHubPluginUpdater {
    
//         private $slug; // plugin slug
//         private $pluginData; // plugin data
//         private $username; // GitHub username
//         private $repo; // GitHub repo name
//         private $pluginFile; // __FILE__ of our plugin
//         private $githubAPIResult; // holds data from GitHub
//         private $accessToken; // GitHub private repo token
    
//         function __construct( $pluginFile, $gitHubUsername, $gitHubProjectName, $accessToken = '' ) {
//             // add_filter( "pre_set_site_transient_update_plugins", array( $this, "setTransient" ) );
//             // add_filter( "plugins_api", array( $this, "setPluginInfo" ), 10, 3 );
//             // add_filter( "upgrader_post_install", array( $this, "postInstall" ), 10, 3 );
    
//             $this->pluginUrl = str_replace(str_replace('\\', '/', ABSPATH),'', dirname($pluginFile));

//             $this->pluginFile = $pluginFile;
//             $this->username = $gitHubUsername;
//             $this->repo = $gitHubProjectName;
//             $this->accessToken = $accessToken;
//         }
    
//         // Get information regarding our plugin from WordPress
//         private function initPluginData() {
//         $this->slug = plugin_basename( $this->pluginFile );
//         $this->pluginData = get_plugin_data( $this->pluginFile );
//         }
    
//         // Get information regarding our plugin from GitHub
//         private function getRepoReleaseInfo() {
//             // Only do this once
//             if ( ! empty( $this->githubAPIResult ) ) {
//                 return;
//             }
    
//             // Query the GitHub API
//             $url = "https://api.github.com/repos/{$this->username}/{$this->repo}/releases";
            
//             // We need the access token for private repos
//             $callArgs = [];
//             if ( ! empty( $this->accessToken ) ) {
//                 $callArgs = [
//                     'headers' => [
//                         'Authorization' => 'token ' . $this->accessToken
//                     ]
//                 ];
//             }
            
//             // Get the results
//             $this->githubAPIResult = wp_remote_retrieve_body( wp_remote_get( $url, $callArgs ) );
//             if ( ! empty( $this->githubAPIResult ) ) {
//                 $this->githubAPIResult = @json_decode( $this->githubAPIResult );
//             }
    
//             // Use only the latest release
//             if ( is_array( $this->githubAPIResult ) ) {
//                 $this->githubAPIResult = $this->githubAPIResult[0];
//             }
//         }
    
//         // Push in plugin version information to get the update notification
//         public function setTransient( $transient ) {
//             // If we have checked the plugin data before, don't re-check
//             if ( empty( $transient->checked ) ) {
//                 return $transient;
//             }
    
//             // Get plugin & GitHub release information
//             $this->initPluginData();
//             $this->getRepoReleaseInfo();
    
//             // Check the versions if we need to do an update
//             if(!isset($this->githubAPIResult->tag_name) ) return;
            
//             $doUpdate = version_compare( $this->githubAPIResult->tag_name, $transient->checked[$this->slug] );
    
//             // Update the transient to include our updated plugin data
//             if ( $doUpdate == 1 ) {
//             $package = $this->githubAPIResult->zipball_url;
    
//             // Include the access token for private GitHub repos
            
//             if ( !empty( $this->accessToken ) ) {

//                 $package = add_query_arg( array( "access_token" => $this->accessToken ), $package );
//             }
    
//             $obj = new PluginInfos();
//             $obj->slug = $this->slug;
//             $obj->new_version = $this->githubAPIResult->tag_name;
//             $obj->url = $this->pluginData["PluginURI"];
//             $obj->package = $package;
//             $obj->icons = array(
//                 "1x" =>     $this->pluginUrl . 'assets/logo-128x128.jpg',
//                 "2x" =>     $this->pluginUrl . 'assets/logo-256x256.jpg'
//             );
//             $obj->banners = array(
//                 "low" =>    $this->pluginUrl . 'assets/banner-772x250.jpg',
//                 "high" =>   $this->pluginUrl . 'assets/banner-1544x500.jpg'
//             );
//             $obj->banners_rtl = array();
    
//             // Gets the tested version of WP if available
//             $matches = null;
//             preg_match( "/tested:\s([\d\.]+)/i", $this->githubAPIResult->body, $matches );
//             if ( ! empty( $matches ) ) {
//                 if ( is_array( $matches ) ) {
//                     if ( count( $matches ) > 1 ) {
//                         $obj->tested = $matches[1];
//                     }
//                 }
//             }
            
//             $transient->response[$this->slug] = $obj;
//             }
//             return $transient;
//         }
    
//         // Push in plugin version information to display in the details lightbox
//         public function setPluginInfo( $false, $action, $response ) {
//             // Get plugin & GitHub release information
//             $this->initPluginData();
//             $this->getRepoReleaseInfo();
//             // If nothing is found, do nothing
//             if ( empty( $response->slug ) || $response->slug != $this->slug ) {
//             return false;
//             }
    
//             // Add our plugin information
//             $response->last_updated = $this->githubAPIResult->published_at;
//             $response->slug = $this->slug;
//             $response->plugin_name  = $this->pluginData["Name"];
//             $response->name  = $this->pluginData["Name"];
//             $response->version = $this->githubAPIResult->tag_name;
//             $response->author = $this->pluginData["AuthorName"];
//             $response->homepage = $this->pluginData["PluginURI"];
//             $response->banners = array(
//                 "high" => $this->pluginUrl . 'assets/banner-1544x500.jpg',
//                 "low" => $this->pluginUrl . 'assets/banner-772x250.jpg'
//             );
            
//             // This is our release download zip file
//             $downloadLink = $this->githubAPIResult->zipball_url;
            
//             // Include the access token for private GitHub repos
//             if ( !empty( $this->accessToken ) ) {
//                 $downloadLink = add_query_arg(
//                     array( "access_token" => $this->accessToken ), $downloadLink
//                 );
//             }
//             $response->download_link = $downloadLink;
    
//             // We're going to parse the GitHub markdown release notes, include the parser
//             require_once( plugin_dir_path( __FILE__ ) . 'class.parsedown.php' );
    
//             // Create tabs in the lightbox
//             $changelogs = class_exists( "Parsedown" )
//                             ? Parsedown::instance()->parse( $this->githubAPIResult->body )
//                             : $this->githubAPIResult->body;
//             $response->sections = array(
//                 'description' => $this->pluginData["Description"],
//                 'changelog' => substr($changelogs,0,strpos($changelogs,'requires:'))
//             );
    
//             // Gets the required version of WP if available
//             $matches = null;
//             preg_match( "/requires:\s([\d\.]+)/i", $this->githubAPIResult->body, $matches );
//             if ( ! empty( $matches ) ) {
//                 if ( is_array( $matches ) ) {
//                     if ( count( $matches ) > 1 ) {
//                         $response->requires = $matches[1];
//                     }
//                 }
//             }
            
//             // Gets the tested version of WP if available
//             $matches = null;
//             preg_match( "/tested:\s([\d\.]+)/i", $this->githubAPIResult->body, $matches );
//             if ( ! empty( $matches ) ) {
//                 if ( is_array( $matches ) ) {
//                     if ( count( $matches ) > 1 ) {
//                         $response->tested = $matches[1];
//                     }
//                 }
//             }
//             return $response;
//         }
    
//         // Perform additional actions to successfully install our plugin
//         public function postInstall( $true, $hook_extra, $result ) {
//             // Get plugin information
//             $this->initPluginData();
    
//             // Remember if our plugin was previously activated
//             $wasActivated = is_plugin_active( $this->slug );
    
//             // Since we are hosted in GitHub, our plugin folder would have a dirname of
//             // reponame-tagname change it to our original one:
//             global $wp_filesystem;
//             $pluginFolder = WP_PLUGIN_DIR . DIRECTORY_SEPARATOR . dirname( $this->slug );
//             $wp_filesystem->move( $result['destination'], $pluginFolder );
//             $result['destination'] = $pluginFolder;
    
//             // Re-activate plugin if needed
//             if ( $wasActivated ) {
//                 $activate = activate_plugin( $this->slug );
//             }
    
//             return $result;
//         }
//     }
// }