/* 
CONTROLLERS
*/
var id3Plugin = angular.module('id3Plugin', ['ngRoute','ngMaterial','ngAnimate', 'ngMessages','ngSanitize','angularMoment']);

id3Plugin
.config(function ($routeProvider, $mdThemingProvider, $httpProvider, $mdDateLocaleProvider, $mdAriaProvider,$animateProvider) {
    $animateProvider.classNameFilter(/angular-animate/);
    // disable aria warnings (aka aria-label warning of hell)
    $mdAriaProvider.disableWarnings();

    // Theming
    $mdThemingProvider.theme('default')
        .primaryPalette('orange')
        .accentPalette('orange');

    $mdThemingProvider.theme('docs-dark', 'default')
        .primaryPalette('blue')
        .dark();

    
});


id3Plugin
.controller('rootController', 
function(
    $scope, $rootScope, $element,
    $q, $timeout, $http,$mdDialog){

    $scope._is_ready = false;
    $scope._current_module = null;
    $scope._auto_save = true;

    $scope.registrations = [];

    $scope.rootInit = function(){
    
        console.log('id3Plugin@rootController/rootInit');

        $scope.fetchRegistrations();
    }

    $rootScope.isReady = function($options=null){
        $options = angular.merge({
            moduleName : null,
            autoSaveOnChange: null,
            serviceList: [$id3UI, $id3Rest,$id3Settings]
        },$options);

        $scope._current_module = $options.moduleName;
        $scope._auto_save = $options.autoSaveOnChange;
        if($scope._auto_save) $id3Settings.saveOnChange($scope._auto_save);

        if($scope._is_ready) return $q.resolve();
        return $q( ($resolve, $reject) => {
            $q.all($options.serviceList.map($s => {
                return $s.isReady()
            }))
            .then(
                _ => {
                    $
                    $resolve();
                },
                _ => {
                    $reject();
                }
            )
            .catch($err => {
                $reject($err);
            });
        });
    }

    $scope.fetchRegistrations = function(){
        $http.get('/wp-json/id3-dlp/registrations').then($response => {
            if($response.status == 200){
                $scope.registrations = $response.data;
            }
        })
    }

    $scope.deleteRegistration = function($key){
        $scope.confirm("Are you sure you want to remove this registration?").then( () => {
            
            $http.delete('/wp-json/id3-dlp/registration?slave_id=' + $key).then($response => {
                if($response.status == 200){
                    $scope.fetchRegistrations();
                }
            })
            
        })
    }

    $scope.deactivateRegistration = function($key){
        $scope.confirm("Are you sure you want to deactivate this registration?").then( () => {
            
            $http.post('/wp-json/id3-dlp/registration/deactivate?slave_id=' + $key).then($response => {
                if($response.status == 200){
                    $scope.fetchRegistrations();
                }
            })
            
        })
    }
    $scope.activateRegistration = function($key){
            
        $http.post('/wp-json/id3-dlp/registration/activate?slave_id=' + $key).then($response => {
            if($response.status == 200){
                $scope.fetchRegistrations();
            }
        })
    
    }

    $rootScope.getSettings  = function(){

    }

    $rootScope.saveSettings = function(){
        
    }

    $rootScope.resetSettings = function(){
        
    }


    /**
   * Display a confirmation box
   * @param {string} $title Main confirm message
   * @param {string} $message Optional. Additionnal information to help understand the main message. Default : empty
   * @param {object} $options Optional. Additionnal options to configure button labels and more. Default : null
   * @return {promise}
   */
  $scope.confirm = function($title, $message, $options){
    const lTitle = ($message == undefined) ? '': $title;
    const lMessage = ($message == undefined) ? $title : $message;

    $options = angular.merge({
      ev: null,
      ok: 'OK',
      cancel: 'Cancel',
      multiple: true
    }, $options);
    
    // Appending dialog to document.body to cover sidenav in docs app
    var confirm = $mdDialog.confirm()
          .title(lTitle)
          .htmlContent(lMessage.replace("\n",'<br>'))
          .multiple($options.multiple)
          .targetEvent($options.ev)
          .ok($options.ok)
          .cancel($options.cancel);

    confirm._options.parent = angular.element(document.body);
    return $mdDialog.show(confirm);
  }

});